<?php

/**
 * Re-organise all uploaded resources for an import to Wordpress
 *
 *  BEWARE: Make sure you back up everything. I can't guarantee this script
 *  won't trash your data, delete your uploaded resources, or eat your dog.
 *
 * Michael Tyson <mike@tyson.id.au>
 * Version 0.1, 5 Sep 2008
 */

/**
 * Configuration
 */

// Database details
$db_host = "localhost";
$db_name = "wordpress";
$db_user = "wordpress";
$db_pass = "password";

// Transformations to apply on original resource url to get original physical resource path relative to current path
$transform_matches = array("|^http://[^/]+|", "|^/*blog/+uploads|", "|^/*uploads|");
$transform_replace = array("", "wp-content/uploads", "wp-content/uploads");

// Pattern to look for to match resources
$resource_pattern = "|(?:http://[^/]+)/?[^'\"]*uploads[^'\"]+|";

// Operation to apply when re-organising: copy or move
$operation = "move";

// Prefix to apply to new path to convert to url
$new_path_url_prefix = "http://".$_SERVER["HTTP_HOST"]."/";

// Path to search for resources if they can't be found immediately
$search_path = ".";

// If resource was found at $search_path, force copy instead of move?
$copy_instead_if_found_elsewhere = true;

// Dry run only, don't make any changes
$dry_run = true; if ( !$_REQUEST["dry-run"] ) $dry_run = false;


if ( !isset($_REQUEST["begin"]) ) {
	?>
	<html>
	<body>
	<h1>Reorganise media</h1>
	<p>
	This utility will re-organise all of the uploaded resources referenced in your Wordpress posts.
	</p>
	<p>
	Before you begin, it is critically important to back up your Wordpress database, and all of your uploaded
	resources, as well as the wp-content/uploads folder.
	</p>
	<p>
	If something goes wrong, then you can restore the database and resources.
	</p>
	<p>
	Edit this file to configure appropriate settings, then ensure "Dry run" is checked to perform a test without making
	any changes. If all is well, press Back in your browser, turn off "Dry run", then proceed.
	</p>
	<h2>Ready?</h2>
	<p>
	<form method="get">
	<input type="checkbox" name="dry-run" checked> Dry run<br/>
	<input type="submit" name="begin" value="Begin now">
	</form>
	</p>
	</body>
	</html>
	<?php
	exit;
}


/**
 * Operation starts here
 */
set_time_limit(0);
ini_set('display_errors', '1');

// Connect to database
mysql_connect($db_host, $db_user, $db_pass) || die("Couldn't connect to database");
mysql_select_db($db_name) || die("Couldn't access database");

// Query posts
$result = mysql_query("SELECT ID, post_title, post_date, post_content FROM wp_posts WHERE post_type = 'post'");

echo "<h1>".mysql_num_rows($result)." entries to process".($dry_run?" (Dry run)":"")."</h1>";

// Loop over posts
$problems = array();
$warnings = array();
$moved_paths = array();
$count = 0;
while (($obj = mysql_fetch_object($result))) {
	$count++;
	
	// Find links to uploaded files
	if ( !preg_match_all($resource_pattern, $obj->post_content, $matches, PREG_OFFSET_CAPTURE) ) continue;
	$matches = $matches[0];
	
	echo "<h2>Post ".$obj->ID.": ".htmlspecialchars($obj->post_title)."</h2>";

	echo "<ul>";
	
	// Loop over found matches
	$updated = false;
	for ( $matchnumber=0; $matchnumber<count($matches); $matchnumber++ ) {
		list($text, $position) = $matches[$matchnumber];
		
		// Determine old path
		$original = urldecode(preg_replace("|//+|", "/", "./".preg_replace($transform_matches, $transform_replace, $text)));
		
		// Determine new path, derived from post date; change these lines if a different path is desired
		$time = strtotime($obj->post_date);
		$new = preg_replace("|//+|", "/", "wp-content/uploads/".date("Y", $time)."/".date("m", $time)."/".basename($original));
		
		if ( $original == "./".$new ) {
			// Link already fine - skip it.
			continue;
		}
		
		if ( file_exists($new) ) {
			echo "<li>$new already exists</li>";
		} else {
			if ( $moved_paths[$original] ) {
				// This path has already been moved. Just link to it
				$new = $moved_paths[$original];
				echo "<li>Path $original already moved; using $new</li>";
			} else {
				
				$elsewhere = false;
				
				if ( !file_exists($original) ) {
					// Can't find original file - try searching for it.
					echo "<li>Searching for $original: <tt>find ".escapeshellarg($search_path)." -name ".escapeshellarg(basename($original))."</tt></li>";
					$results = explode("\n", shell_exec("find ".escapeshellarg($search_path)." -name ".escapeshellarg(basename($original))));
					if ( !$results || !trim($results[0]) ) {
						echo "<li><b>Couldn't find $original!</b></li>";
						$errors[] = array($obj->ID, "Couldn't find resource $original");
						continue;
					}
					echo "<li>Found at ".$results[0]."</li>";
					$warnings[] = array($obj->ID, "Found $original at ".$results[0]);
					$original = trim($results[0]);
					$elsewhere = true;
				}
				
				$moved_paths[$original] = $new;
				
				if ( $original != "./".$new ) {
	
					$dirname = dirname($new);
					if ( !file_exists($dirname) ) {
						echo "<li>Creating $dirname</li>";
						if ( !$dry_run ) {
							if ( !mkdir($dirname, 0755, true) ) {
								echo "<li><b>Creation of $dirname failed</b></li>";
								$errors[] = array($obj->ID, "Couldn't create directory $dirname");
								continue;
							}
						}
					}
					
					if ( file_exists($new) ) {
						// If destination file already exists, just link to that instead and move/copy original to a temp folder
						echo "<li>$new already exists. Will ".($operation=="move"?"move":"copy")." $original into temporary folder 'reorganise-media-tmp'</li>";
						$warnings[] = array($obj->ID, "$new already exists. Will ".($operation=="move"?"move":"copy")." $original into temporary folder 'reorganise-media-tmp'");
						if ( !file_exists("reorganise-media-tmp") ) {
							echo "<li>Creating reorganise-media-tmp</li>";
							if ( !$dry_run ) {
								if ( !mkdir("reorganise-media-tmp", 0755, true) ) {
									echo "<li><b>Creation of reorganise-media-tmp failed. $original will remain where it is</b></li>";
									$errors[] = array($obj->ID, "Couldn't create directory reorganise-media-tmp. $original will remain where it is");
								}
							}
						}
						if ( file_exists("reorganise-media-tmp") ) {
							if ( $operation == "copy" || ($elsewhere && $copy_instead_if_found_elsewhere)) {
								echo "<li>Copying $original to reorganise-media-tmp/".basename($original)."</li>";
								if ( !$dry_run ) $opResult = copy($original, "reorganise-media-tmp/".basename($original));
							} else if ( $operation == "move" ) {
								echo "<li>Moving $original to reorganise-media-tmp/".basename($original)."</li>";
								if ( !$dry_run ) $opResult = rename($original, "reorganise-media-tmp/".basename($original));
							}
	
							if ( !$dry_run && !$opResult ) {
								echo "<li><b>Operation failed</b></li>";
								$errors[] = array($obj->ID, "Failed ".($operation=="move"?"moving":"copying")." $original to $new");
								continue;
							}
						}
					} else {
						if ( $operation == "copy" || ($elsewhere && $copy_instead_if_found_elsewhere) ) {
							echo "<li>Copying $original to $new</li>";
							if ( !$dry_run ) $opResult = copy($original, $new);
						} else if ( $operation == "move" ) {
							echo "<li>Moving $original to $new</li>";
							if ( !$dry_run ) $opResult = rename($original, $new);
						}
	
						if ( !$dry_run && !$opResult ) {
							echo "<li><b>Operation failed</b></li>";
							$errors[] = array($obj->ID, "Failed ".($operation=="move"?"moving":"copying")." $original to $new");
							continue;
						}
					}
				}
			}
		}
		
		// Update post content to reflect new resource location
		$new = $new_path_url_prefix.$new;
		$obj->post_content = substr_replace($obj->post_content, $new, $position, strlen($text));
		$updated = true;
		
		// Update subsequent matches to reflect altered content
		for ( $i=$matchnumber+1; $i<count($matches); $i++ ) {
			$matches[$i][1] += strlen($new)-strlen($text);
		}
	}
	
	if ( $dry_run ) {
		echo "<blockquote>".htmlspecialchars($obj->post_content)."</blockquote>";
	}
	
	if ( !$dry_run && $updated ) {
		// Perform action
		echo "<li>Updating post</li>";
		$updateResult = mysql_query("UPDATE wp_posts SET post_content = '".mysql_escape_string($obj->post_content)."' WHERE ID = '".mysql_escape_string($obj->ID)."'");
		if ( !$updateResult ) {
			$err = mysql_error($updateResult);
			echo "<li><b>Couldn't update database for post ".$obj->ID.": $err</b></li>";
			$errors[] = array($obj->ID, "Couldn't update database: $err");
		}
	}
	
	echo "</ul>";
}

echo "<h1>Summary</h1>";

if ( $errors ) {
	echo "<h2>Errors:</h2><ul>";
	foreach ( $errors as $error ) {
		echo "<li>Post ".$error[0].": ".$error[1]."</li>";
	}
	echo "</ul>";
}

if ( $warnings ) {
	echo "<h2>Warnings:</h2><ul>";
	foreach ( $warnings as $warning ) {
		echo "<li>Post ".$warning[0].": ".$warning[1]."</li>";
	}
	echo "</ul>";
}

echo "<p>$count items processed.</p>";



?>